local Entity = import('/lua/modules/entity.lua').Entity

oldUnit = Unit
Unit = Class(oldUnit) {
    OnCreate = function(self) 
		oldUnit.OnCreate(self)

		local bp = self:GetBlueprint()
		-- initialise a table to store each modules stats
		if bp.DamagableModules then

			if not self.ModuleStats then
				self.ModuleStats = {}
			end
			for i = 1, table.getn(bp.DamagableModules) do
				if not self.ModuleStats[i] then
					self.ModuleStats[i] = {}
					self.ModuleStats[i].CurrentHealth = bp.DamagableModules[i].Health 
					self.ModuleStats[i].ChanceOfHit = bp.DamagableModules[i].ChanceOfHit
					self.ModuleStats[i].FXTable = {}
					self.ModuleStats[i].Enabled = true
				end
			end 
		end		
    end,

    OnDamage = function(self, instigator, amount, vector, damageType)

        if self.CanTakeDamage then
            self:DoOnDamagedCallbacks(instigator)
            self:DoCalculateDamage(instigator, amount, vector, damageType)
        end
    end,
 
    -- Add smoke and fire effects to the bones listed in the blueprint.
    AddModuleDestroyedEffect = function(self, imodule)
		local bp = self:GetBlueprint()

		local FXCount = 1
		if bp.DamagableModules[imodule].FXCount then
			FXCount = bp.DamagableModules[imodule].FXCount 
		end

		local FXScale = 1
		if bp.DamagableModules[imodule].FXScale then
			FXScale = bp.DamagableModules[imodule].FXScale
		end
	
		local boneTable = bp.DamagableModules[imodule].FXBones
		if not boneTable then boneTable = bp.DamagableModules[imodule].Bones end
		-- apply damage effect to a random selection of bones listed
		if boneTable then
			local army = self:GetArmy()
        		local effects1 = self.FxDamage1[Random(1,table.getn(self.FxDamage1))]
			local effects2 = self.FxDamage2[Random(1,table.getn(self.FxDamage2))]
        		if not effects1 or not effects2 then return end
			for i = 1, FXCount do
        			for k, v in effects1 do
            			local fx
					local random_bone = boneTable[Random(1, table.getn(boneTable))]
                			fx = CreateAttachedEmitter(self, random_bone, army, v):ScaleEmitter(FXScale)
					table.insert(self.ModuleStats[imodule].FXTable, fx)
        			end
        			for k, v in effects2 do
            			local fx
					local random_bone = boneTable[Random(1, table.getn(boneTable))]
                			fx = CreateAttachedEmitter(self, random_bone, army, v):ScaleEmitter(FXScale)
					table.insert(self.ModuleStats[imodule].FXTable, fx)
        			end
			end
		end
    end,

    -- Apply damage to given module
    DoTakeModuleDamage = function(self, instigator, amount, vector, damageType, imodule)
		local bp = self:GetBlueprint()

		-- Apply damage to module
		self.ModuleStats[imodule].CurrentHealth = self.ModuleStats[imodule].CurrentHealth - amount
		
		-- destroy module if it has health less than 0
		if self.ModuleStats[imodule].CurrentHealth < 0 then
			LOG('Module ', bp.DamagableModules[imodule].Label, ' destroyed')
			self.ModuleStats[imodule].CurrentHealth = 0 
			self.ModuleStats[imodule].Enabled = false
			self:AddModuleDestroyedEffect(imodule)
		
			if self.DamagableModules[bp.DamagableModules[imodule].Label].OnDestroy then
				self.DamagableModules[bp.DamagableModules[imodule].Label].OnDestroy(self)
			end
		end
    end,

    DoCalculateDamage = function(self, instigator, amount, vector, damageType)
		local bp = self:GetBlueprint()
		
		local hitModules = {}

		if bp.DamagableModules then
			local rootBonePos = self:GetPosition()
			local damagePos = {rootBonePos[1] - vector[1], rootBonePos[2] - vector[2], rootBonePos[3] - vector[3]} 
			for imodule = 1, table.getn(bp.DamagableModules) do
				local chanceOfHit = bp.DamagableModules[imodule].ChanceOfHit
				if not chanceOfHit then chanceOfHit = 0.3 end	-- default chance if none specified
				
				-- Do we have specified damage bones?
				if bp.DamagableModules[imodule].Bones then
					-- is damage spot within radius of any of the bones?
					local boneRadius = bp.DamagableModules[imodule].Radius
					if not boneRadius then boneRadius = 1 end	-- bon radius is the same for every bone
					local moduleHit = false
					for i, bone in bp.DamagableModules[imodule].Bones do
						local bonePos = self:GetPosition(bone)
						if VDist3(bonePos, damagePos) < boneRadius then
							moduleHit = true
						end
					end
					
					-- were any of the bones close enough to the damage to get hit?
					if moduleHit and self.ModuleStats[imodule].Enabled then
						LOG('Module ', bp.DamagableModules[imodule].Label, ' hit')
						if chanceOfHit > math.random() then
							LOG('Module ', bp.DamagableModules[imodule].Label, ' damaged')
							self:DoTakeModuleDamage(instigator, amount, vector, damageType, imodule)
						end
					end
				else
					-- no damagble bones specified so figure out if module was hit by chance
					if chanceOfHit > math.random()  and self.ModuleStats[imodule].Enabled then
						LOG('Module ', bp.DamagableModules[imodule].Label, ' damaged by chance')
						self:DoTakeModuleDamage(instigator, amount, vector, damageType, imodule)
					end
				end
			end
		end
		
		-- Damage the main unit regardless of whether a module was damaged or exists
		self:DoTakeDamage(instigator, amount, vector, damageType)

    end,
    
    OnHealthChanged = function(self, new, old)
	-- This reports a change in the units health. If it is going up (ie being repaired)
	-- then we need to increase each module's health proportionaly as well.
	local bp = self:GetBlueprint()

	if bp.DamagableModules then
		if new > old then
			-- Unit is being repaired
			local bp = self:GetBlueprint()

			local percentIncrease = new - old

			for i = 1, table.getn(bp.DamagableModules) do
				local healthIncrease = bp.DamagableModules[i].Health * percentIncrease
				self.ModuleStats[i].CurrentHealth = self.ModuleStats[i].CurrentHealth + healthIncrease

				-- Check if module needs reenabled
				if not self.ModuleStats[i].Enabled and self.ModuleStats[i].CurrentHealth > bp.DamagableModules[i].Health * 0.1 then
					LOG('Renabling Module ', bp.DamagableModules[i].Label)
					self.ModuleStats[i].Enabled = true
					# remove damage effects
					for k, v in self.ModuleStats[i].FXTable do
						v:Destroy()
					end

					-- call enable callbacks
					if self.DamagableModules[bp.DamagableModules[i].Label].OnEnable then
						self.DamagableModules[bp.DamagableModules[i].Label].OnEnable(self)
					end
				end

				-- Check max health hasn't been exceeded
				if self.ModuleStats[i].CurrentHealth > bp.DamagableModules[i].Health then
					self.ModuleStats[i].CurrentHealth = bp.DamagableModules[i].Health 
				end
			end
		end
	else
		-- no damagable modules defined so treat like a regular unit
      	self:ManageDamageEffects(new, old)
	end
    end,
}
