#****************************************************************************
#**
#**  File     :  /cdimage/units/UES0302/UES0302_script.lua
#**  Author(s):  John Comes, David Tomandl, Jessica St. Croix
#**
#**  Summary  :  UEF Battleship Script
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local TSeaUnit = import('/lua/modules/terranunits.lua').TSeaUnit
local WeaponsFile = import('/lua/modules/terranweapons.lua')
local TAALinkedRailgun = WeaponsFile.TAALinkedRailgun
local TAMPhalanxWeapon = WeaponsFile.TAMPhalanxWeapon
local TDFGaussCannonWeapon = WeaponsFile.TDFGaussCannonWeapon

UES0302 = Class(TSeaUnit) {
    DamagableModules = {
	  	FrontTurret01 = {
			OnDestroy = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('FrontTurret01', false)
			end,
			OnEnable = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('FrontTurret01', true)
			end,
		},
	  	FrontTurret02 = {
			OnDestroy = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('FrontTurret02', false)
			end,
			OnEnable = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('FrontTurret02', true)
			end,
		},
	  	BackTurret = {
			OnDestroy = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('BackTurret', false)
			end,
			OnEnable = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('BackTurret', true)
			end,
		},
	  	RightAAGun01 = {
			OnDestroy = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('RightAAGun01', false)
			end,
			OnEnable = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('RightAAGun01', true)
			end,
		},
	  	RightAAGun02 = {
			OnDestroy = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('RightAAGun02', false)
			end,
			OnEnable = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('RightAAGun02', true)
			end,
		},
	  	LeftAAGun01 = {
			OnDestroy = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('LeftAAGun01', false)
			end,
			OnEnable = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('LeftAAGun01', true)
			end,
		},
	  	LeftAAGun02 = {
			OnDestroy = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('LeftAAGun02', false)
			end,
			OnEnable = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('LeftAAGun02', true)
			end,
		},
	  	RightPhalanxGun01 = {
			OnDestroy = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('RightPhalanxGun01', false)
			end,
			OnEnable = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('RightPhalanxGun01', true)
			end,
		},
	  	LeftPhalanxGun01 = {
			OnDestroy = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('LeftPhalanxGun01', false)
			end,
			OnEnable = function(parent_unit) 
				parent_unit:SetWeaponEnabledByLabel('LeftPhalanxGun01', true)
			end,
		},
    },

    Weapons = {
        RightAAGun01 = Class(TAALinkedRailgun) {},
        RightAAGun02 = Class(TAALinkedRailgun) {},
        LeftAAGun01 = Class(TAALinkedRailgun) {},
        LeftAAGun02 = Class(TAALinkedRailgun) {},
        RightPhalanxGun01 = Class(TAMPhalanxWeapon) {
            PlayFxWeaponUnpackSequence = function(self)
                if not self.SpinManip then 
                    self.SpinManip = CreateRotator(self.unit, 'Right_Turret02_Barrel', 'z', nil, 270, 180, 60)
                    self.unit.Trash:Add(self.SpinManip)
                end
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(270)
                end
                TAMPhalanxWeapon.PlayFxWeaponUnpackSequence(self)
            end,

            PlayFxWeaponPackSequence = function(self)
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(0)
                end
                TAMPhalanxWeapon.PlayFxWeaponPackSequence(self)
            end,
            
        },
        LeftPhalanxGun01 = Class(TAMPhalanxWeapon) {
            PlayFxWeaponUnpackSequence = function(self)
                if not self.SpinManip then 
                    self.SpinManip = CreateRotator(self.unit, 'Left_Turret02_Barrel', 'z', nil, 270, 180, 60)
                    self.unit.Trash:Add(self.SpinManip)
                end
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(270)
                end
                TAMPhalanxWeapon.PlayFxWeaponUnpackSequence(self)
            end,

            PlayFxWeaponPackSequence = function(self)
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(0)
                end
                TAMPhalanxWeapon.PlayFxWeaponPackSequence(self)
            end,

        },
        FrontTurret01 = Class(TDFGaussCannonWeapon) {
            FxMuzzleFlash = {
                '/effects/emitters/cannon_muzzle_fire_01_emit.bp',
                '/effects/emitters/gauss_cannon_muzzle_flash_02_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_03_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_04_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_05_emit.bp',
                '/effects/emitters/cannon_muzzle_water_shock_01_emit.bp',
            },
            FxMuzzleFlashScale = 0.8,
            },
        FrontTurret02 = Class(TDFGaussCannonWeapon) {
            FxMuzzleFlash = {
                '/effects/emitters/cannon_muzzle_fire_01_emit.bp',
                '/effects/emitters/gauss_cannon_muzzle_flash_02_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_03_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_04_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_05_emit.bp',
                '/effects/emitters/cannon_muzzle_water_shock_01_emit.bp',
            },
            FxMuzzleFlashScale = 0.8,
            },
        BackTurret = Class(TDFGaussCannonWeapon) {
            FxMuzzleFlash = {
                '/effects/emitters/cannon_muzzle_fire_01_emit.bp',
                '/effects/emitters/gauss_cannon_muzzle_flash_02_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_03_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_04_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_05_emit.bp',
                '/effects/emitters/cannon_muzzle_water_shock_01_emit.bp',
            },
            FxMuzzleFlashScale = 0.8,
            },
    },

    OnStopBeingBuilt = function(self,builder,layer)
        TSeaUnit.OnStopBeingBuilt(self,builder,layer)
        self.Trash:Add(CreateRotator(self, 'Spinner01', 'y', nil, -45))
        self.Trash:Add(CreateRotator(self, 'Spinner02', 'y', nil, 90))
    end,
}

TypeClass = UES0302